/*
 *  WHO LET THE BLOCKS OUT?
 *
 *  This is yet another block game where the objective
 *  is to line up three or more matching blocks. The goal is not 
 *  based upon clearing the well, but obtaining the highest score
 *  in the time limit. The challenge lays in attempting to rack up
 *  the points by clearing the blocks as soon as they hit the 
 *  ground.
 *  
 *  Programmed by Matthew Leverton (matthew@allegro.cc) for the
 *  January 2001 Allegro Speedhack competition.
 *
 *  Feel free to use any or all of this source code in your own
 *  game, as long as you give credit where it is due.
 *
 *  Questions and comments pertaining to the source are welcomed.
 *
 *  http://members.allegro.cc/matthew/
 *
 */

/******************************************************************
 * main.c
 *
 * Initializes the game, and contains the main logic loop.
 *
 */

#include <time.h>

#include "allegro.h"

#include "main.h"
#include "data.h"
#include "player.h"
#include "animate.h"
#include "hiscore.h"
#include "advert.h"

/* #define DEBUG 1 */

/* timer for controlling game speed */
volatile UINT32 counter;

#ifdef DEBUG
	volatile UINT8 fps_counter = 0;
	volatile UINT32 fps = 0;
	volatile UINT32 fps_drawn = 0;
#endif

static void timer(void)
{
	counter++;   
	#ifdef DEBUG
		if (++fps_counter == 60)
		{
			fps_counter = 0;
			fps = fps_drawn;
			fps_drawn = 0;
		}
	#endif 
}

END_OF_STATIC_FUNCTION(timer);

DATAFILE *data;

INT32 C_WHITE;
INT32 C_BLACK;
INT32 C_YELLOW;
INT32 clock_ticks;


void draw_box(BITMAP *bmp, INT16 left, INT16 top, INT16 width, INT16 height)
{
	/* just a cheesy routine to draw transparent square boxes with a border */
	set_trans_blender(0,0,0,128);
	drawing_mode(DRAW_MODE_TRANS, NULL, 0,0);
	
	width--;
	height--;
	
	rectfill(bmp,left+4,top+4, left+width-4,top+height-4,makecol(0,0,0));
	rect(bmp,left+3,top+3, left+width-3,top+height-3,makecol(0,0,0));
	rect(bmp,left+2,top+2, left+width-2,top+height-2,makecol(255,255,255));
	rect(bmp,left+1,top+1, left+width-1,top+height-1,makecol(255,255,255));
	rect(bmp,left,top,     left+width,top+height,makecol(0,0,0));
}


INT8 play_game()
{	
	BITMAP *bmp_buffer = create_bitmap(SCREEN_W, SCREEN_H);
	BITMAP *bmp_back = create_bitmap(SCREEN_W,SCREEN_H);
	BITMAP *bmp_title = create_bitmap(SCREEN_W,SCREEN_H);
	UINT32 k;
	INT8 menu;
	
	BYTE draw_something = 0;
	UINT32 logic_counter;

	/* just for the fun of it */
	play_sample((SAMPLE*)data[WAV_BEEP].dat,255,127,1000,0);

	/* the title screen */	
	clear(bmp_buffer);
	stretch_blit((BITMAP *)data[BMP_TITLE].dat, bmp_title, 0,0, ((BITMAP *)data[BMP_TITLE].dat)->w,((BITMAP *)data[BMP_TITLE].dat)->h, 0,0, 640,480);
	stretch_blit((BITMAP *)data[BMP_PICTURE].dat, bmp_back, 0,0,((BITMAP *)data[BMP_PICTURE].dat)->w,((BITMAP *)data[BMP_PICTURE].dat)->h, 0,0, 640,480);
	for (k=0; k<640; k+=2)
		line(bmp_back,0,k,639,k,C_BLACK);

	
	blit(bmp_title,screen,0,0,0,0,640,480);		
	draw_box(bmp_back, 280,124,80,40);

	/* and draw the menu */
	text_mode(-1);
	menu = 2;

	/* a lame menu system */
	do {
		textprintf(screen,font,60,350,menu == 1 ? C_YELLOW : C_WHITE,"[1] One Player Game");
		textprintf(screen,font,60,366,menu == 2 ? C_YELLOW : C_WHITE,"[2] Two Player Game");
		textprintf(screen,font,60,382,menu == 3 ? C_YELLOW : C_WHITE,"[ESC] Exit the Game");
		clear_keybuf();
		k = readkey() >> 8;
		/* handle the key press ... 666 is used as a chosen marker */
		switch (k)
		{
			case KEY_ESC: menu = 3; k = 666; break;
			case KEY_1: menu = 1; k = 666; break;
			case KEY_2: menu = 2; k = 666; break;
			case KEY_UP: if (--menu < 1) menu = 3; break;
			case KEY_DOWN: if (++menu > 3) menu = 1; break;
			case KEY_ENTER: k = 666; break;			
		}
	} while (k != 666);

	if (menu == 3) return 0;

	/* initialize players and well positions */
	player_init(menu);
	player_well_pos(0, bmp_buffer, 64,48);
	draw_box(bmp_back, 60,44,200,392);
	if (menu == 2) 
	{
		player_well_pos(1, bmp_buffer, 384,48);	
		draw_box(bmp_back, 380,44,200,392);
	}
	blit(bmp_back,bmp_buffer,0,0,0,0,640,480);

	/* music to start the game */
	play_midi((MIDI*) data[MIDI_THEME].dat,1);
	play_sample((SAMPLE*)data[WAV_BEGIN].dat,255,127,1000,0);

	clock_ticks = 7200; /* two minutes */
	logic_counter = counter;
	
	clear_keybuf(); 
	do 
	{
		/* spacebar will pause in debug mode */
		#ifdef DEBUG
			while (key[KEY_SPACE]) logic_counter = counter;			
		#endif

		/* always maintain 60 frames of logic per second */
		while (logic_counter < counter)
		{
			if (player_move()) clock_ticks = 0;
			animate_move();
			draw_something = 1;
			logic_counter++;	
			if (clock_ticks) clock_ticks--;
			if (clock_ticks == 900) play_sample((SAMPLE*)data[WAV_TICKS].dat,255,127,1000,0);
			if (clock_ticks == 30) play_sample((SAMPLE*)data[WAV_WHISTLE].dat,255,127,1000,0);
		}
		
		/* now lets try to draw 60 fps, but never any more than needed */
		if (draw_something || key[KEY_B])
		{
			#ifdef DEBUG 
				fps_drawn++;
			#endif
			player_well_draw(bmp_buffer);
			player_draw(bmp_buffer, bmp_back);			
			animate_draw(bmp_buffer);
			if (key[KEY_V]) vsync();
			blit(bmp_buffer,screen,0,0,0,0,SCREEN_W-1,SCREEN_H-1);				
		
			draw_something = 0;
		}
		
		/* show the fps in debug mode */
		#ifdef DEBUG 
			text_mode(0);
			textprintf(bmp_buffer,font,0,0,makecol(255,255,255),"FPS: %i  ",fps);		
		#endif
	} while (!key[KEY_ESC] && clock_ticks);
	
	stop_midi();

	player_end_game_draw(bmp_buffer);
	blit(bmp_buffer,screen,0,0,0,0,SCREEN_W-1,SCREEN_H-1);
	rest(3000);


	/* show the high scores and stuff */
	player_add_scores();
	hiscore_draw();

	destroy_bitmap(bmp_back);
	destroy_bitmap(bmp_buffer);
	destroy_bitmap(bmp_title);


	player_shutdown();

	return 1;
}

int main(int argc, char *argv[])
{
	/* set up the wonderful allegro library */
	allegro_init();
	install_timer();
	install_keyboard();
	install_mouse();
	install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL);	

	/* setup the timers */
	LOCK_VARIABLE(counter);
	#ifdef DEBUG
		LOCK_VARIABLE(fps_counter);
		LOCK_VARIABLE(fps);
		LOCK_VARIABLE(fps_drawn);
	#endif
	LOCK_FUNCTION(timer); 
	counter = 0;
	install_int_ex(timer, BPS_TO_TIMER(60));
	srand(time(NULL));

	/* install the graphics mode. we are assuming at least 1 works! */	
	set_color_depth(16);
	if (set_gfx_mode(GFX_AUTODETECT,640,480,0,0))
	{
		set_color_depth(15);
		set_gfx_mode(GFX_AUTODETECT,640,480,0,0);
	} 
	C_WHITE = makecol(255,255,255);
	C_BLACK = makecol(0,0,0);
	C_YELLOW = makecol(255,255,192);

	/* now we can load the global data file & initialize a few more things */
	data = load_datafile("blocks.dat");		
	hiscore_init();
	animate_init();

	/* let's show the advertisement */
	advertisement();	
	
	/* hey, they're playing the game! */
	while (play_game());
	
	/* and just in case they forgot... */
	advertisement();	

	hiscore_shutdown();
	
	return 0;
}
END_OF_MAIN();