/******************************************************************
 * well.c
 *
 * Controls all the various aspects of the player's well. The 
 * drawing routines are terribly slow, as they use brute force to
 * update the screen....
 *
 */

#include "allegro.h"
#include "data.h"
#include "main.h"
#include "well.h"
#include "chain.h"
#include "block.h"
#include "player.h"

BITMAP *bmp_blocks[6];

void well_draw(WELL *w)
{	
	BYTE r, c;
	CHAIN *chain = w->chains; 
	BLOCK *block;
	BITMAP *bmp = w->bmp_well;
	
	/* YIKES! this is __really__ unoptimized */	

	blit(w->bmp_backdrop, bmp, 0, 0, 0, 0, 192, 384); 
	set_trans_blender(0, 0, 0, 192);
	
	/* draw main blocks */
	for (r=0; r!=ROWS; r++)
		for (c=0; c!=COLS; c++)
		{			
			if (w->block[r][c].tile && w->block[r][c].status != S_CHAIN) draw_trans_sprite(bmp, bmp_blocks[w->block[r][c].tile],  c*32+w->block[r][c].c_adjust, r*32-w->scroll_offset+w->block[r][c].r_adjust);
		
			#ifdef DEBUG
			textprintf(bmp,font,4+c*32+w->block[r][c].c_adjust, 12+r*32-w->scroll_offset+w->block[r][c].r_adjust,makecol(255,255,255),"%i",w->block[r][c].status);
			textprintf(bmp,font,90,170,makecol(255,0,0),"%i",w->skill_chains);
			#endif
		}

	/* color chains (ie, blocks that are being destroyed) */
	while (chain)
	{
		if (chain->type == C_DESTROY)
		{
			set_trans_blender(0, 0, 0, 48+chain->grace);
			block = chain->blocks;
			while (block)
			{
				draw_trans_sprite(bmp, bmp_blocks[w->block[block->r][block->c].tile], block->c*32, block->r*32-w->scroll_offset);
				block = block->next;
			}
		}
		chain = chain->next;
	}

	set_trans_blender(0, 0, 0, 224);	

	/* draw new blocks @ the bottom */
	for (c=0; c!=COLS; c++)
		draw_trans_sprite(bmp, bmp_blocks[w->block[ROWS][c].tile],  c*32, r*32-w->scroll_offset);

	/* draw selector */
	draw_sprite(bmp, w->bmp_selector[(counter/16)%2], w->sx*32-6, w->sy*32-6-w->scroll_offset);
		
}

void well_init(WELL *w)
{
	BITMAP *bmp_temp;
	BYTE r, c;

	/* clear the well */
	for (r=0; r!=ROWS; r++)
		for (c=0; c!=COLS; c++)
		{
			w->block[r][c].tile = 0;
			w->block[r][c].status = 0;
			w->block[r][c].r_adjust = 0;
			w->block[r][c].c_adjust = 0;
		}

	/* fill half of the well with random blocks */
	for (r=ROWS/2; r!=ROWS+1; r++)	
		for (c=0; c!=COLS; c++)
		{
			BYTE b = (rand() % 5) + 1;

			/* we've got to make sure that no 3 consecutive blocks (horizontal or vertical) are the same */
			while ((c>1 && w->block[r][c-1].tile == b && w->block[r][c-2].tile == b) || (r>1 && w->block[r-1][c].tile == b && w->block[r-2][c].tile == b))
				b = (rand() % 5) + 1;

			w->block[r][c].tile = b;
			w->block[r][c].status = 0;
			w->block[r][c].r_adjust = 0;
			w->block[r][c].c_adjust = 0;
		}

	/* load default bitmaps */
	w->bmp_backdrop = create_bitmap(192,384);
	stretch_blit((BITMAP *)data[BMP_DONKEY].dat, w->bmp_backdrop, 0,0,((BITMAP *)data[BMP_DONKEY].dat)->w,((BITMAP *)data[BMP_DONKEY].dat)->h,0,0,192,384); 
	
	bmp_temp = create_bitmap(640,32);
	draw_sprite(bmp_temp, (BITMAP *)data[BMP_BLOCKS].dat,0,0);

	for (c=0; c!=6; c++)
	{
		bmp_blocks[c] = create_bitmap(32,32);
		clear(bmp_blocks[c]);
		blit(bmp_temp, bmp_blocks[c], c*32,0, 0,0, 32,32);
	}
	destroy_bitmap(bmp_temp);
	w->bmp_selector[0] = (BITMAP*) data[BMP_SELECTOR1].dat;
	w->bmp_selector[1] = (BITMAP*) data[BMP_SELECTOR2].dat;
	
	/* initialize the top left coordinates of the well. */
	w->dx = 0;
	w->dy = 0;

	/* initialize the coordinates for the selector */
	w->sx = COLS/2-1;
	w->sy = ROWS/2-1;

	/* initialize the scrolling */
	w->scroll_wait = 0;
	w->scroll_offset = 10;
	w->scroll_speed = 60;
	w->scroll_counter = 60;

	w->chains = NULL;
	w->skill_chains = 0;
}


void well_shutdown(WELL *w)
{
	/* we won't be needing these anymore */
	destroy_bitmap(w->bmp_backdrop);
	destroy_bitmap(w->bmp_well);
}